/* autosts.c */
/* Ian Scott (ian@norcimo.com)*/
/* v3.0 (April 2003) */

/* You may freely copy, redistribute and modify this code for non-commercial purposes provided you acknowledge the original author and make freely available, under identical conditions and restrictions, all code thus produced  */


/* This programme is based on Mark Hadley's original sts.c programme. Quite a bit has been changed since then, though a lot of the underlying mechanism is still the same */

/*
autosts.c : A program to perform (dI/dV)/(I/V) spectroscopy and output
resulting values. The I/V may be broadened in a variety of ways.
Performs this for several files with the files name convention s# where # is an positive integer numbering the files sequentially, or a specified file list
Input file should consist of a file with < MAXPT lines, each containing
one pair of V,I. There should be no header lines.
*/

#include <stdio.h>
#include <math.h>
#include "stsallutil.h"
/* This file contains broadening, differention functions, etc. */
#include <string.h>
#include <stdlib.h>

/* #define DEBUG */

extern void *malloc();
extern void spint(float *x,float *f,int n,float *ix,float *ifn,int in);
extern void polyint(float *x,float *f,int n,float *ix,float *ifn,int in,int order);

void main(int argc, char *argv[])
{
float **V,**I;
float **IV,**dIdV,**tx,**IVraw,**IVoff,**didvtmp;
float smoothed[256];
float c=0.05;
/* factor for offset broadening */
int i,j,nc,*ndp,*nddp,columns,rows,specno,specmax;
int flags=0, offset=0;
/* Keeps track of various switches */
short *lines,*points;
FILE *infp,*fp;
unsigned short plots;
char outname[255], rname[255],k[2],tempy[255];

/* Allocate memory */
V=malloc(sizeof(float *));
I=malloc(sizeof(float *));
IV=malloc(sizeof(float *));
IVoff=malloc(sizeof(float *));
IVraw=malloc(sizeof(float *));
dIdV=malloc(2*sizeof(float *));
didvtmp=malloc(sizeof(float *));
tx=malloc(2*sizeof(float *));
ndp=malloc(sizeof(int));
nddp=malloc(2*sizeof(int));
points=malloc(2*sizeof(short));
lines=malloc(2*sizeof(short));
V[0]=malloc(MAXPT*sizeof(float));
I[0]=malloc(MAXPT*sizeof(float));
IV[0]=malloc(MAXPT*sizeof(float));
IVraw[0]=malloc(MAXPT*sizeof(float));
IVoff[0]=malloc(MAXPT*sizeof(float));
dIdV[0]=malloc(2*MAXPT*sizeof(float));
didvtmp[0]=malloc(MAXPT*sizeof(float));
tx[0]=malloc(2*MAXPT*sizeof(float));

nc=1;
tx[nc]=tx[nc-1]+MAXPT;
dIdV[nc]=dIdV[nc-1]+MAXPT;

/* Need to determine which switches have been set
Each switch sets a bit of the integer flags as follows:
bit 0: loop  switch: -l
bit 1: no broadening switch: -n
bit 2: convolute with exponential (original method) switch: -e
bit 3: use offset c (cf UKSPM 03) switch: -c
bit 4: use -n -e -c switch: -a
bit 5: suppress output other than normalised switch: -s
bit 7: display help switch: -h

*/
switches(&flags, &offset, argc, argv);
#ifdef DEBUG
printf("flags: %i; offset: %i\n\n",flags,offset);
#endif

if ((argc-offset-1==0) && !(flags & 1) && !(flags & 128))
	{
	printf("\nDon't know which files to process!\n");
	flags|=128;
	}

if (flags & 128)
	{
	printf("\nThis is autosts v3.0\n\n");
	printf("Usage:\n\n");
	/* find filename, no extension or path */
	while(strchr(argv[0],'\\'))
		argv[0]=strchr(argv[0],'\\')+1;
	if (strchr(argv[0],'.'))
		argv[0]=strtok(argv[0],".");
	printf("%s [switches] [filelist]", argv[0]);
	printf("\n\nThe following switches are supported:\n");
	printf("-l Loop sequentially over files s#, # being an integer\n");
	printf("-n Output raw result (no broadening etc)\n");
	printf("-e Output result with broadening by convolution with exponential\n");
	printf("-c Output result with broadening by offset\n");
	printf("-a Equivalent to -n -e -c (Default)\n");
	printf("-s Suppress output other than normalised curve\n");
	printf("-h Display this help\n\n");
	exit(0);
	}

if (!flags || flags==1 || flags==17 || flags==16)
	flags|=14; /* Defaults to all outputs */

printf("\nAutosts 3.0 running...\n\n");

if (flags & 1)
	{
        printf("Enter number of spectroscopy files to process: ");
        scanf("%d",&specmax);
        if (specmax<1)
        	exit(0);
        printf("\nProcessing %d spectroscopy files...\n",specmax);
        }

if (!(flags & 1))
	specmax=argc-offset-1;

for (specno=1;specno<=specmax;specno++)
        {
        if (flags & 1)
        	{
        	strcpy(outname,"s");
        	strcat(outname,ims_itoa(specno));
        	}
        else
        	{
        	strcpy(outname,argv[specno+offset]);
        	}
        j=0;

	if ((infp = fopen(outname,"r")) == NULL)
		{
		printf("Can't open %s; skipping..\n",outname);
		continue;
		}

	/* Read in input data */
	i=(-1);
	do
		{
		i++;
		} while (fscanf(infp,"%f%f",V[j]+i,I[j]+i)==2);
	ndp[j]=i;
	printf("%s: %d data points read",outname,ndp[j]);
	fclose(infp);
	sort2(ndp[j],V[j]-1,I[j]-1);

	lines[0]=points[0]=1;

/* Reset Zero of data
Makes sure the I-Vcurve goes through the point (0,0). */
	j=0;
	setzero(V[j],I[j],ndp[j]);

	/* calculate I/V */
	j=0;
	for(i=0;i<ndp[j];i++)
		IV[j][i]=I[j][i]/V[j][i];
#ifdef DEBUG
printf("I/V has been calculated\n");
#endif

	/* Differentiate data */
	j=0;

	diff(V[j],I[j],dIdV[j],tx[j],ndp[j]);
	/* NB this differentiation routine could
	in principle do anything
	 one less point after differentiation */
	nddp[j]=ndp[j]-1;

/* output didv against v */

	if (flags & 1)
		{
		strcpy(rname,"stxt");
		strcat(rname,ims_itoa(specno));
		strcat(rname,"didv.txt");
		}
	else
		{
		strcpy(rname,argv[specno+offset]);
		strcat(rname,"didv.txt");
		}
#ifdef DEBUG
printf("output filename is: %s\n",rname);
#endif
	if (!(flags & 16))
		{
        	fp=fopen(rname,"w");
        	if (!fp)
        		{
        		printf("Couldn't open di/dv output file. Exit\n\n");
        		exit(1);
        		}
        	for (i=0;i<nddp[j];i++)
        		{
        		/* di/dv against v */
        		fprintf(fp,"%f %s %f",tx[j][i]," ",dIdV[j][i]);
        		fprintf(fp,"\n");
        		}
        	fclose(fp);
        	}

	for (i=0;i<ndp[j];i++)
		{
		IVraw[j][i]=IV[j][i];
		IVoff[j][i]=IV[j][i];
		}
		/* Three different things happen to I/V so stored 3 times */
#ifdef DEBUG
for (i=0;i<ndp[j];i++)
{
printf("IV: %f ;",IV[j][i]);
printf("IVoff: %f\n",IVoff[j][i]);
}
#endif
/* exponential broaden I/V */
	j=0;
	broaden(V[j],IV[j],(float)1.33,ndp[j]);

/* c offset I/V */
	for (i=0;i<ndp[j];i++)
		{

		IVoff[j][i]=sqrt(IVoff[j][i]*IVoff[j][i]+c*c);
#ifdef DEBUG
printf("  IVoff/sqrt(IVoff^2+c^2): %f\n",IVoff[j][i]);
#endif
		}

/* Output !! */

	if (flags & 1)
		{
		strcpy(rname,"stxt");
		strcat(rname,ims_itoa(specno));
		strcat(rname,"iv.txt");
		}
	else
		{
		strcpy(rname, argv[specno+offset]);
		strcat(rname,"iv.txt");
		}
	if (!(flags & 16))
		{
        	fp=fopen(rname,"w");
        	for (i=0;i<ndp[j];i++)
        		{
        		/* i/v against v */
        		fprintf(fp,"%f %s %f",V[j][i]," ",IVraw[j][i]);
        		fprintf(fp,"\n");
        		}
        	fclose(fp);
        	}

	if (flags & 1)
		{
		strcpy(rname,"stxt");
		strcat(rname,ims_itoa(specno));
		strcat(rname,"i.txt");
		}
	else
		{
		strcpy(rname, argv[specno+offset]);
		strcat(rname,"i.txt");
		}
	if (!(flags & 16))
		{
        	fp=fopen(rname,"w");
        	for (i=0;i<ndp[j];i++)
        		{
        		/* i against v */
        		fprintf(fp,"%f %s %f",V[j][i]," ",I[j][i]);
        		fprintf(fp,"\n");
        		}
        	fclose(fp);
        	}


 	/* Normalise data */

 	/* Exponential */
 	for (i=0;i<nddp[j];i++)
 		didvtmp[j][i]=dIdV[j][i];
 #ifdef DEBUG
 printf("dIdV[0]: %f ",dIdV[j][0]);
 printf("tx[0]: %f\n",tx[j][0]);
 printf("V[0]: %f\n",V[j][0]);
 printf("nddp: %i\n",nddp[j]);
 printf("ndp: %i\n",ndp[j]);
 printf("IV[0] %f\n",IV[j][0]);
 #endif
	normalise(tx[j],dIdV[j],nddp[j],V[j],IV[j],ndp[j]);
	if (flags & 1)
		{
		strcpy(rname,"stxt");
		strcat(rname,ims_itoa(specno));
		strcat(rname,"exp.txt");
		}
	else
		{
		strcpy(rname,argv[specno+offset]);
		strcat(rname,"exp.txt");
		}
	if (flags & 4)
		{
        	fp=fopen(rname,"w");
        	for (i=0;i<nddp[j];i++)
        		{
        		/* di/dv/(i/v) raw data against v */
        		fprintf(fp,"%f %s %f",tx[j][i]," ",dIdV[j][i]);
        		fprintf(fp,"\n");
        		}
        	fclose(fp);
        	}

	for (i=0;i<nddp[j];i++)
 		dIdV[j][i]=didvtmp[j][i];

	/* C offset */
#ifdef DEBUG
printf("dIdV[0]: %f ",dIdV[j][0]);
printf("tx[0]: %f\n",tx[j][0]);
 printf("V[0]: %f\n",V[j][0]);
 printf("nddp: %i\n",nddp[j]);
 printf("ndp: %i\n",ndp[j]);
  printf("IVoff[0] %f\n",IVoff[j][0]);
#endif
	normalise(tx[j],dIdV[j],nddp[j],V[j],IVoff[j],ndp[j]);
	if (flags & 1)
		{
		strcpy(rname,"stxt");
		strcat(rname,ims_itoa(specno));
		strcat(rname,"coff.txt");
		}
	else
		{
		strcpy(rname,argv[specno+offset]);
		strcat(rname,"coff.txt");
		}
	if (flags & 8)
		{
        	fp=fopen(rname,"w");
        	for (i=0;i<nddp[j];i++)
        		{
        		/* di/dv/(i/v) raw data against v */
        		fprintf(fp,"%f %s %f",tx[j][i]," ",dIdV[j][i]);
        		fprintf(fp,"\n");
        		}
        	fclose(fp);
        	}

	for (i=0;i<nddp[j];i++)
 		dIdV[j][i]=didvtmp[j][i];

	/* raw */

	normalise(tx[j],dIdV[j],nddp[j],V[j],IVraw[j],ndp[j]);
	if (flags & 1)
		{
		strcpy(rname,"stxt");
		strcat(rname,ims_itoa(specno));
		strcat(rname,"raw.txt");
		}
	else
		{
		strcpy(rname,argv[specno+offset]);
		strcat(rname,"raw.txt");
		}
	if (flags & 2)
		{
        	fp=fopen(rname,"w");
        	for (i=0;i<nddp[j];i++)
        		{
        		/* di/dv/(i/v) raw data against v */
        		fprintf(fp,"%f %s %f",tx[j][i]," ",dIdV[j][i]);
        		fprintf(fp,"\n");
        		}
        	fclose(fp);
        	}
        printf(" ..done\n");
        }
printf("\nAll files done!!\n\n");
free(V[0]);
free(I[0]);
free(IV[0]);
free(IVraw[0]);
free(IVoff[0]);
free(dIdV[0]);
free(didvtmp[0]);
free(tx[0]);
free(V);
free(I);
free(IV);
free(IVraw);
free(IVoff);
free(dIdV);
free(didvtmp);
free(tx);
free(ndp);
free(nddp);
free(points);
free(lines);
}