/* Collection of all the utility functions used in sts program
(some stolen from numerical recipes in c */

/* You may freely copy, redistribute and modify this code for non-commercial purposes provided you acknowledge the original author and make freely available, under identical conditions and restrictions, all code thus produced  */

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "stsallutil.h"

void nrerror(char error_text[])
{
	void exit();

	fprintf(stderr,"Numerical Recipes run-time error...\n");
	fprintf(stderr,"%s\n",error_text);
	fprintf(stderr,"...now exiting to system...\n");
	exit(1);
}

float *vector(int nl, int nh)
{
	float *v;

	v=(float *)malloc((unsigned) (nh-nl+1)*sizeof(float));
	if (!v) nrerror("allocation failure in vector()");
	return v-nl;
}

void free_vector(float *v, int nl, int nh)
{
	free((char*) (v+nl));
}


void setzero(float x[],float f[],int n)
{
    float tmp[MAXPT],dy;
    int i;

    spline(x-1,f-1,n,(float)1.0e30,(float)1.0e30,tmp-1);
    splint(x-1,f-1,tmp-1,n,(float)0.0,&dy);
    for(i=0;i<n;i++)
        f[i]-=dy;
}

void broaden(float x[],float f[],float dx,int n)
{
#define PAD 10
    int i,arrsize;
    float *src,*respons,*ans;
    float polyx[3],polyy[3];

    for(i=0;(arrsize=1<<i)<=n;i++);
    while(n+2*PAD > arrsize-2*PAD) arrsize*=2;
    src=malloc(arrsize*sizeof(float));
    respons=malloc(arrsize*sizeof(float));
    ans=malloc(2*arrsize*sizeof(float));

    for(i=0;i<arrsize;i++)
        src[i]=respons[i]=0.0;
    for(i=0;i<n;i++)
        src[i+PAD]=f[i];
    polyx[0]=x[0];polyx[1]=x[n/2];polyx[2]=x[n-1];
    polyy[0]=f[0];polyy[1]=f[n/2];polyy[2]=f[n-1];
    for(i=0;i<PAD;i++)
    {
        float delta,err;

        delta=(i+1)*(x[1]-x[0]);
        polint(polyx-1,polyy-1,3,(float)(x[0]-delta),&src[PAD-i-1],&err);
        polint(polyx-1,polyy-1,3,(float)(x[n-1]+delta),&src[n+PAD+i],&err);
    }
    for(i=0;i<(n+1)/2;i++)
        respons[i]=respons[n-i]=exp(i*(x[0]-x[1])/dx);
    respons[n]=0.0;
    convlv(src-1,arrsize,respons-1,n,1,ans-1);
    for(i=0;i<n;i++)
        f[i]=ans[i+PAD];
    free(src);
    free(respons);
    free(ans);
#undef PAD
}

void diff(float x[],float f[],float dfdx[],float nx[],int n)
{
/* x is an array containing the x points */
/* y contains the y points */
/* dfdx is the array into which the diffed points */
/* are placed. nx contains the x values for these */
/*points. n is the number of original points */

    int i;

    for(i=1;i<n;i++)
        dfdx[i-1]=(f[i]-f[i-1])/(x[i]-x[i-1]);
    for(i=0;i<n-1;i++)
        nx[i]=(x[i]+x[i+1])/2.0;
}

void normalise(float x[],float f[],int n,float nx[],float nf[],int nn)
{
    float tmp[MAXPT];
    int i;

    spline(nx-1,nf-1,nn,(float)1.0e30,(float)1.0e30,tmp-1);
    for(i=0;i<n;i++)
    {
        float val;

        splint(nx-1,nf-1,tmp-1,nn,x[i],&val);
        f[i]/=val;
    }
}

void splint(float *xa, float *ya, float *y2a, int n, float x,float *y)
{
	int klo,khi,k;
	float h,b,a;
	void nrerror();

	klo=1;
	khi=n;
	while (khi-klo > 1) {
		k=(khi+klo) >> 1;
		if (xa[k] > x) khi=k;
		else klo=k;
	}
	h=xa[khi]-xa[klo];
	if (h == 0.0) nrerror("Bad XA input to routine SPLINT");
	a=(xa[khi]-x)/h;
	b=(x-xa[klo])/h;
	*y=a*ya[klo]+b*ya[khi]+((a*a*a-a)*y2a[klo]+(b*b*b-b)*y2a[khi])*(h*h)/6.0;
}

void spline(float *x, float *y, int n, float yp1, float ypn, float *y2)
{
	int i,k;
	float p,qn,sig,un,*u,*vector();

	u=vector(1,n-1);
	if (yp1 > 0.99e30)
		y2[1]=u[1]=0.0;
	else {
		y2[1] = -0.5;
		u[1]=(3.0/(x[2]-x[1]))*((y[2]-y[1])/(x[2]-x[1])-yp1);
	}
	for (i=2;i<=n-1;i++) {
		sig=(x[i]-x[i-1])/(x[i+1]-x[i-1]);
		p=sig*y2[i-1]+2.0;
		y2[i]=(sig-1.0)/p;
		u[i]=(y[i+1]-y[i])/(x[i+1]-x[i]) - (y[i]-y[i-1])/(x[i]-x[i-1]);
		u[i]=(6.0*u[i]/(x[i+1]-x[i-1])-sig*u[i-1])/p;
	}
	if (ypn > 0.99e30)
		qn=un=0.0;
	else {
		qn=0.5;
		un=(3.0/(x[n]-x[n-1]))*(ypn-(y[n]-y[n-1])/(x[n]-x[n-1]));
	}
	y2[n]=(un-qn*u[n-1])/(qn*y2[n-1]+1.0);
	for (k=n-1;k>=1;k--)
		y2[k]=y2[k]*y2[k+1]+u[k];
	free_vector(u,1,n-1);
}

void sort2(int n, float ra[], float rb[])

{
	int l,j,ir,i;
	float rrb,rra;

	l=(n >> 1)+1;
	ir=n;
	for (;;) {
		if (l > 1) {
			rra=ra[--l];
			rrb=rb[l];
		} else {
			rra=ra[ir];
			rrb=rb[ir];
			ra[ir]=ra[1];
			rb[ir]=rb[1];
			if (--ir == 1) {
				ra[1]=rra;
				rb[1]=rrb;
				return;
			}
		}
		i=l;
		j=l << 1;
		while (j <= ir)	{
			if (j < ir && ra[j] < ra[j+1]) ++j;
			if (rra < ra[j]) {
				ra[i]=ra[j];
				rb[i]=rb[j];
				j += (i=j);
			}
			else j=ir+1;
		}
		ra[i]=rra;
		rb[i]=rrb;
	}
}

void polint(float xa[],float ya[],int n,float x,float *y,float *dy)
{
	int i,m,ns=1;
	float den,dif,dift,ho,hp,w;
	float *c,*d,*vector();

	dif=fabs(x-xa[1]);
	c=vector(1,n);
	d=vector(1,n);
	for (i=1;i<=n;i++) {
		if ( (dift=fabs(x-xa[i])) < dif) {
			ns=i;
			dif=dift;
		}
		c[i]=ya[i];
		d[i]=ya[i];
	}
	*y=ya[ns--];
	for (m=1;m<n;m++) {
		for (i=1;i<=n-m;i++) {
			ho=xa[i]-x;
			hp=xa[i+m]-x;
			w=c[i+1]-d[i];
			if ( (den=ho-hp) == 0.0) nrerror("Error in routine POLINT");
			den=w/den;
			d[i]=hp*den;
			c[i]=ho*den;
		}
		*y += (*dy=(2*ns < (n-m) ? c[ns+1] : d[ns--]));
	}
	free_vector(d,1,n);
	free_vector(c,1,n);
}

static float sqrarg;
#define SQR(a) (sqrarg=(a),sqrarg*sqrarg)

void convlv(float data[], int n, float respns[], int m, float isign, float ans[])
{
	int i,no2;
	float dum,mag2,*fft,*vector();

	fft=vector(1,2*n);
	for (i=1;i<=(m-1)/2;i++)
		respns[n+1-i]=respns[m+1-i];
	for (i=(m+3)/2;i<=n-(m-1)/2;i++)
		respns[i]=0.0;
	twofft(data,respns,fft,ans,n);
	no2=n/2;
	for (i=2;i<=n+2;i+=2) {
		if (isign == 1) {
			ans[i-1]=(fft[i-1]*(dum=ans[i-1])-fft[i]*ans[i])/no2;
			ans[i]=(fft[i]*dum+fft[i-1]*ans[i])/no2;
		} else if (isign == -1) {
 		        if ((mag2=SQR(ans[i-1])+SQR(ans[i])) == 0.0)
				nrerror("Deconvolving at response zero in CONVLV");
			ans[i-1]=(fft[i-1]*(dum=ans[i-1])+fft[i]*ans[i])/mag2/no2;
			ans[i]=(fft[i]*dum-fft[i-1]*ans[i])/mag2/no2;
		} else nrerror("No meaning for ISIGN in CONVLV");
	}
	ans[2]=ans[n+1];
	realft(ans,no2,-1);
	free_vector(fft,1,2*n);
}

#undef SQR

void twofft(float data1[], float data2[],float fft1[], float fft2[], int n)
{
	int nn3,nn2,jj,j;
	float rep,rem,aip,aim;

	nn3=1+(nn2=2+n+n);
	for (j=1,jj=2;j<=n;j++,jj+=2) {
		fft1[jj-1]=data1[j];
		fft1[jj]=data2[j];
	}
	four1(fft1,n,1);
	fft2[1]=fft1[2];
	fft1[2]=fft2[2]=0.0;
	for (j=3;j<=n+1;j+=2) {
		rep=0.5*(fft1[j]+fft1[nn2-j]);
		rem=0.5*(fft1[j]-fft1[nn2-j]);
		aip=0.5*(fft1[j+1]+fft1[nn3-j]);
		aim=0.5*(fft1[j+1]-fft1[nn3-j]);
		fft1[j]=rep;
		fft1[j+1]=aim;
		fft1[nn2-j]=rep;
		fft1[nn3-j] = -aim;
		fft2[j]=aip;
		fft2[j+1] = -rem;
		fft2[nn2-j]=aip;
		fft2[nn3-j]=rem;
	}
}

void realft(float data[], int n, int isign)
{
	int i,i1,i2,i3,i4,n2p3;
	float c1=0.5,c2,h1r,h1i,h2r,h2i;
	double wr,wi,wpr,wpi,wtemp,theta;

	theta=3.141592653589793/(double) n;
	if (isign == 1) {
		c2 = -0.5;
		four1(data,n,1);
	} else {
		c2=0.5;
		theta = -theta;
	}
	wtemp=sin(0.5*theta);
	wpr = -2.0*wtemp*wtemp;
	wpi=sin(theta);
	wr=1.0+wpr;
	wi=wpi;
	n2p3=2*n+3;
	for (i=2;i<=n/2;i++) {
		i4=1+(i3=n2p3-(i2=1+(i1=i+i-1)));
		h1r=c1*(data[i1]+data[i3]);
		h1i=c1*(data[i2]-data[i4]);
		h2r = -c2*(data[i2]+data[i4]);
		h2i=c2*(data[i1]-data[i3]);
		data[i1]=h1r+wr*h2r-wi*h2i;
		data[i2]=h1i+wr*h2i+wi*h2r;
		data[i3]=h1r-wr*h2r+wi*h2i;
		data[i4] = -h1i+wr*h2i+wi*h2r;
		wr=(wtemp=wr)*wpr-wi*wpi+wr;
		wi=wi*wpr+wtemp*wpi+wi;
	}
	if (isign == 1) {
		data[1] = (h1r=data[1])+data[2];
		data[2] = h1r-data[2];
	} else {
		data[1]=c1*((h1r=data[1])+data[2]);
		data[2]=c1*(h1r-data[2]);
		four1(data,n,-1);
	}
}

#define SWAP(a,b) tempr=(a);(a)=(b);(b)=tempr

void four1(float data[], int nn, int isign)
{
	int n,mmax,m,j,istep,i;
	double wtemp,wr,wpr,wpi,wi,theta;
	float tempr,tempi;

	n=nn << 1;
	j=1;
	for (i=1;i<n;i+=2) {
		if (j > i) {
			SWAP(data[j],data[i]);
			SWAP(data[j+1],data[i+1]);
		}
		m=n >> 1;
		while (m >= 2 && j > m) {
			j -= m;
			m >>= 1;
		}
		j += m;
	}
	mmax=2;
	while (n > mmax) {
		istep=2*mmax;
		theta=6.28318530717959/(isign*mmax);
		wtemp=sin(0.5*theta);
		wpr = -2.0*wtemp*wtemp;
		wpi=sin(theta);
		wr=1.0;
		wi=0.0;
		for (m=1;m<mmax;m+=2) {
			for (i=m;i<=n;i+=istep) {
				j=i+mmax;
				tempr=wr*data[j]-wi*data[j+1];
				tempi=wr*data[j+1]+wi*data[j];
				data[j]=data[i]-tempr;
				data[j+1]=data[i+1]-tempi;
				data[i] += tempr;
				data[i+1] += tempi;
			}
			wr=(wtemp=wr)*wpr-wi*wpi+wr;
			wi=wi*wpr+wtemp*wpi+wi;
		}
		mmax=istep;
	}
}

#undef SWAP

char* ims_itoa(int integer)
{
int number = 0, i = 0, j = 0;
char tempNum,textNum[255];

if (integer >= 0) { number = integer; }
else { number = -integer; }

/* convert integer values to characters from right to left */

do {
    textNum[j++] = (number % 10) + '0';
    number /= 10;
} while (number != 0);

if (integer < 0) { textNum[j++] = '-'; }

textNum[j--] = '\0';

/* reverse the array since it was written backwards */

while (i < j) {
    tempNum = textNum[i];
    textNum[i++] = textNum[j];
    textNum[j--] = tempNum;
}

return textNum;
}

void switches(int *flags, int *offset, int argc, char* argv[])
{
int fin=0;
int i=1;
char f;

*offset=0;
*flags=0;

while (!fin && i<argc)
	{
	if (argv[i][0]=='-')
		{
		f=argv[i][1];
		switch (f)
			{
			case 'l':
				{
				*flags|=1;
				break;
				}
			case 'n':
				{
				*flags|=2;
				break;
				}
			case 'e':
				{
				*flags|=4;
				break;
				}
			case 'c':
				{
				*flags|=8;
				break;
				}
			case 'a':
				{
				*flags|=14;
				break;
				}
			case 's':
				{
				*flags|=16;
				break;
				}
			case 'h':
				{
				*flags|=128;
				break;
				}
			default:
				{
				*offset=*offset-1;
				fin=1;
				}
			}
		*offset=*offset+1;
		}
	else
		{
		fin=1;
		}
	i++;
	}
}